@@ -1,5 +1,5 @@
 /* Nabi - X Input Method server for hangul
- * Copyright (C) 2007 Choe Hwanjin
+ * Copyright (C) 2007-2008 Choe Hwanjin
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,8 +48,12 @@ nabi_log(int level, const char* format, ...)
     if (level <= log_level) {
 	va_list ap;
 
+	fprintf(output_device, "Nabi(%d): ", level);
+
 	va_start(ap, format);
 	vfprintf(output_device, format, ap);
 	va_end(ap);
+
+	fflush(output_device);
     }
 }
